# Movable Type Link plugin (C) 2010 Alfasado Inc.
# This program is distributed under the terms of the
# GNU General Public License, version 2.

package MT::Plugin::Link;
use strict;
use MT;
use MT::Plugin;
use Link::Link;
use File::Temp qw( tempdir ); 
use Encode;
use MT::Util qw( format_ts offset_time_list encode_js encode_html );
use MT::I18N qw( substr_text length_text );
our $VERSION = '1.01';
our $SCHEMA_VERSION = '0.29';
use base qw( MT::Plugin );

@MT::Plugin::Link::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::Link( {
    id   => 'Link',
    key  => 'link',
    name => 'Link',
    description => '<__trans phrase="Manage Link object.">',
    author_name => 'Alfasado Inc.',
    author_link => 'http://alfasado.net/',
    version => $VERSION,
    schema_version => $SCHEMA_VERSION,
    settings => new MT::PluginSettings( [
        [ 'check_outlink', { Default => 1 } ],
    ] ),
    blog_config_template => 'link_config.tmpl',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        applications => {
            cms => {
                menus => {
                    'link' => {
                        label => 'Link',
                        order => 450,
                        condition  => \&_link_permission,
                    },
                    'link:list_link' => {
                        label => 'Manage',
                        mode  => 'list_link',
                        order => 1,
                        condition  => \&_link_permission,
                        view => [ 'website', 'blog' ],
                    },
                    'link:create_link' => {
                        label => 'New',
                        mode  => 'view',
                        args  => { _type => 'link', },
                        order => 2,
                        condition  => \&_link_permission,
                        view => [ 'website', 'blog' ],
                    },
                    'link:edit_tags' => {
                        label => 'Tags',
                        mode  => 'list_tag',
                        args  => { filter_key => 'link', },
                        order => 5,
                        permission => 'edit_tags',
                        view => [ 'website', 'blog' ],
                    },
                },
                methods => {
                    list_link =>
                        'MT::Plugin::Link::_list_link',
                    upload_link =>
                        'MT::Plugin::Link::_upload_link',
                    publish_links =>
                        'MT::Plugin::Link::_publish_links',
                    unpublish_links =>
                        'MT::Plugin::Link::_unpublish_links',
                    action_link_check =>
                        'MT::Plugin::Link::_action_link_check',
                    view_link =>
                        'MT::Plugin::Link::_view_link',
                },
                search_apis => {
                    link =>
                      { handler => 'MT::Plugin::Link::_search_link',
                        condition => sub {
                            return 1 if MT->instance()->blog;
                        },
                        perm_check => \&_link_permission,
                        order => 700,
                        label => 'Link',
                        can_replace => 0,
                        can_search_by_date => 1,
                        date_column => 'authored_on',
                        search_cols => {
                            'name' => sub { $plugin->translate( 'Name' ) },
                            'notes' => sub { $plugin->translate( 'Notes' ) },
                            'description' => sub { $plugin->translate( 'Description' ) },
                            'url' => sub { $plugin->translate( 'URL' ) },
                            'rss_address' => sub { $plugin->translate( 'RSS URL' ) },
                            'image_address' => sub { $plugin->translate( 'Image URL' ) },
                            'title' => sub { $plugin->translate( 'title' ) },
                        },
                        setup_terms_args => sub {
                            my ( $terms, $args, $blog_id ) = @_;
                            $terms->{ blog_id } = $blog_id if $blog_id;
                        },
                        results_table_template => sub {
                            my $path = $plugin->path;
                            my $tmpl = qq(<mt:include name="$path/tmpl/include/link_table.tmpl">);
                            return $tmpl;
                        },
                    },
                },
            },
        },
        object_types => {
            'link' => 'Link::Link',
        },
        permissions => {
            'blog.manage_link' => {
                label => 'Link Administrator',
                group => 'blog_admin',
                order => 1000,
            },
        },
        callbacks => {
            'MT::App::CMS::pre_run'
                => \&_pre_run,
            'MT::App::CMS::template_param.edit_link'
                => \&_edit_link,
            'MT::App::CMS::template_param.list_tag'
                => \&_list_tag,
            'MT::App::CMS::template_source.archetype_editor'
                => \&_archetype_editor,
        },
        upgrade_functions => {
            'install_role' => {
                plugin => 'Link',
                # version_limit => $SCHEMA_VERSION,
                code => \&_install_role,
            }
        },
        tasks => {
            'scheduled_link_check' => {
                label     => 'Link check task',
                frequency => 1,
                code      => \&_task_link_check,
                priority  => 5,
            },
        },
        tags => {
            block => {
                'Links'       => \&_hdlr_links,
                'LinksHeader' => \&_hdlr_pass_tokens,
                'LinksFooter' => \&_hdlr_pass_tokens,
                'LinkTags'    => \&_hdlr_link_tags,
                'LinkAuthor'  => \&_hdlr_link_author,
                'LinkIfTagged?'  => \&_hdlr_if_link_tagged,
                'IfBrokenLink?'  => \&_hdlr_if_link_broken,
                'IfBrokenRss?'   => \&_hdlr_if_link_broken,
                'IfBrokenImage?' => \&_hdlr_if_link_broken,
                'IfActiveLink?'  => \&_hdlr_if_link_active,
                'IfActiveRss?'   => \&_hdlr_if_link_active,
                'IfActiveImage?' => \&_hdlr_if_link_active,
            },
            function => {
                'LinkBlogID' => \&_hdlr_link_column,
                'LinkName'   => \&_hdlr_link_column,
                'LinkHTML'   => \&_hdlr_link_html,
                'LinkAuthorDisplayName' => \&_hdlr_author_displayname,
                'LinkUrl' => \&_hdlr_link_column,
                'LinkDescription' => \&_hdlr_link_column,
                'LinkTitle' => \&_hdlr_link_column,
                'LinkTarget' => \&_hdlr_link_column,
                'LinkRel' => \&_hdlr_link_column,
                'LinkImageAddress' => \&_hdlr_link_column,
                'LinkRSSAddress' => \&_hdlr_link_column,
                'LinkNotes' => \&_hdlr_link_column,
                'LinkRating' => \&_hdlr_link_column,
                'LinkAuthoredOn' => \&_hdlr_link_date,
                'LinkCreatedOn'  => \&_hdlr_link_date,
                'LinkModifiedOn' => \&_hdlr_link_date,
                'LinkURLUpdatedOn' => \&_hdlr_link_date,
                'LinkRSSUpdatedOn' => \&_hdlr_link_date,
            },
        },
   } );
}

sub _view_link {
    my $app = shift;
    $app->{ plugin_template_path } = File::Spec->catdir( $plugin->path, 'tmpl' );
    $app->mode( 'edit' );
    return $app->forward( 'edit', @_ );
}

sub _pre_run {
    my ( $cb, $app ) = @_;
    if ( ( $app->mode eq 'save' ) && ( $app->param( '_type' ) eq 'link' ) ) {
        my $id = $app->param( 'id' );
        if ( $id ) {
            my $original = MT->model( 'link' )->load( $id );
            $original = $original->clone_all() if $original;
            MT::Request->instance->cache( 'link_original' . $id, $original );
        }
    }
}

sub _hdlr_links {
    my ( $ctx, $args, $cond ) = @_;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $blog = $ctx->stash( 'blog' );
    my $blog_id = $args->{ blog_id };
    if ( $blog_id && ( $blog_id != $blog->id ) ) {
        $blog = MT::Blog->load( $blog_id );
    }
    $blog_id = $blog->id if (! $blog_id );
    my $lastn  = $args->{ lastn };
    my $limit  = $args->{ limit };
    $limit = $lastn if $lastn;
    my $offset = $args->{ offset };
    my $sort_order = $args->{ sort_order };
    $sort_order = 'ascend' unless $sort_order;
    $limit = 9999 unless $limit;
    my $sort_by = $args->{ sort_by };
    $sort_by = 'id' unless $sort_by;
    $offset = 0 unless $offset;
    my $rating = $args->{ rating };
    my $tag_name = $args->{ tag };
    my $url_active = $args->{ url_active };
    my $rss_active = $args->{ rss_active };
    my $image_active = $args->{ image_active };
    my %terms;
    my %params;
    $terms{ blog_id } = $blog_id;
    $terms{ status } = Link::Link::RELEASE();
    $terms{ rating } = $rating if $rating;
    if ( $url_active ) {
        $terms{ broken_link } = { not => 1 };
        $terms{ url } = { not => '' };
    }
    if ( $rss_active ) {
        $terms{ broken_rss } = { not => 1 };
        $terms{ rss_address } = { not => '' };
    }
    if ( $image_active ) {
        $terms{ broken_image } = { not => 1 };
        $terms{ image_address } = { not => '' };
    }
    my $more = $args->{ more };
    my $less = $args->{ less };
    if (! $rating ) {
        if ( $more ) {
            $more--;
            $terms{ rating } = { '>' => $more };
        }
        if ( $less ) {
            $less++;
            $terms{ rating } = { '<' => $less };
        }
    }
    if ( $tag_name ) {
        require MT::Tag;
        my $tag = MT::Tag->load( { name => $tag_name }, { binary => { name => 1 } } );
        return '' unless $tag;
        $params{ limit }     = $limit;
        $params{ offset }    = $offset;
        $params{ direction } = $sort_order;
        $params{ 'sort' }    = $sort_by;
        require MT::ObjectTag;
        $params { 'join' } = [ 'MT::ObjectTag', 'object_id',
                   { tag_id  => $tag->id,
                     blog_id => $blog_id,
                     object_datasource => 'link' }, ];
    } else {
        $params{ limit }     = $limit;
        $params{ offset }    = $offset;
        $params{ direction } = $sort_order;
        $params{ sort }      = $sort_by;
    }
    my @links = MT->model( 'link' )->load( \%terms, \%params );
    my $i = 0; my $res = '';
    my $odd = 1; my $even = 0;
    for my $link ( @links ) {
        local $ctx->{ __stash }{ 'link' } = $link;
        my $othor_blog;
        if ( $blog->id != $link->blog_id ) {
            $othor_blog = $link->blog;
        }
        local $ctx->{ __stash }{ 'blog' } = $othor_blog if $othor_blog;
        local $ctx->{ __stash }{ 'blog_id' } = $othor_blog->id if $othor_blog;
        local $ctx->{ __stash }->{ vars }->{ __first__ } = 1 if ( $i == 0 );
        local $ctx->{ __stash }->{ vars }->{ __counter__ } = $i + 1;
        local $ctx->{ __stash }->{ vars }->{ __odd__ } = $odd;
        local $ctx->{ __stash }->{ vars }->{ __even__ } = $even;
        local $ctx->{ __stash }->{ vars }->{ __last__ } = 1 if ( !defined( $links[ $i + 1 ] ) );
        my $out = $builder->build( $ctx, $tokens, {
            %$cond,
            'linksheader' => $i == 0,
            'linksfooter' => !defined( $links[ $i + 1 ] ),
        } );
        if ( !defined( $out ) ) { return $ctx->error( $builder->errstr ) };
        $res .= $out;
        if ( $odd == 1 ) { $odd = 0 } else { $odd = 1 };
        if ( $even == 1 ) { $even = 0 } else { $even = 1 };
        $i++;
    }
    return $res;
}

sub _hdlr_pass_tokens {
    my ( $ctx, $args, $cond ) = @_;
    $ctx->stash( 'builder' )->build( $ctx, $ctx->stash( 'tokens' ), $cond );
}

sub _hdlr_link_tags {
    my ( $ctx, $args, $cond ) = @_;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    require MT::ObjectTag;
    require MT::Asset;
    my $glue = $args->{ glue };
    local $ctx->{ __stash }{ tag_max_count } = undef;
    local $ctx->{ __stash }{ tag_min_count } = undef;
    local $ctx->{ __stash }{ all_tag_count } = undef;
    local $ctx->{ __stash }{ class_type } = 'link';
    my $iter = MT::Tag->load_iter( undef, { 'sort' => 'name',
                                            'join' => MT::ObjectTag->join_on( 'tag_id',
                                                    { object_id => $link->id,
                                                      blog_id => $link->blog_id,
                                                      object_datasource => 'link' },
                                                    { unique => 1 } ) } );
    my $builder = $ctx->stash( 'builder' );
    my $tokens = $ctx->stash( 'tokens' );
    my $res = '';
    while ( my $tag = $iter->() ) {
        next if $tag->is_private && !$args->{ include_private };
        local $ctx->{ __stash }{ Tag } = $tag;
        local $ctx->{ __stash }{ tag_count } = undef;
        local $ctx->{ __stash }{ tag_link_count } = undef;
        defined( my $out = $builder->build( $ctx, $tokens, $cond ) )
            or return $ctx->error( $builder->errstr );
        $res .= $glue if defined $glue && length( $res ) && length( $out );
        $res .= $out;
    }
    return $res;
}

sub _hdlr_link_author {
    my ( $ctx, $args, $cond ) = @_;
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    $ctx->stash( 'author', $link->author );
    $ctx->stash( 'builder' )->build( $ctx, $ctx->stash( 'tokens' ), $cond );
}

sub _hdlr_if_link_tagged {
    my ( $ctx, $args, $cond ) = @_;
    return unless $ctx;
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    my $tag = defined $args->{ name } ? $args->{ name } : ( defined $args->{ tag } ? $args->{ tag } : '' );
    if ( $tag ne '' ) {
        $link->has_tag( $tag );
    } else {
        my @tags = $link->tags;
        @tags = grep /^[^@]/, @tags
            if !$args->{ include_private };
        return @tags ? 1 : 0;
    }
}

sub _hdlr_link_html {
    my ( $ctx, $args, $cond ) = @_;
    return unless $ctx;
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    my $name = $link->name;
    my $url = $link->url;
    if ( (! $name ) || (! $url ) ) {
        return '';
    }
    my $title = $link->title;
    my $target = $link->target;
    my $rel = $link->rel;
    $name = encode_html( $name );
    my $tag = "<a href=\"$url\"";
    if ( $title ) {
        $title = encode_html( $title );
        $tag .= " title=\"$title\"";
    }
    if ( $target ) {
        $target = encode_html( $target );
        $tag .= " target=\"$target\"";
    }
    if ( $rel ) {
        $rel = encode_html( $rel );
        $tag .= " rel=\"$rel\"";
    }
    $tag .= ">$name</a>";
    return $tag;
}

sub _hdlr_link_column {
    my ( $ctx, $args, $cond ) = @_;
    return unless $ctx;
    my $tag = $ctx->stash( 'tag' );
    $tag =lc( $tag );
    $tag =~ s/^link//i;
    $tag = 'blog_id' if $tag eq 'blogid';
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    return $link->$tag || '';
}

sub _hdlr_link_date {
    my ( $ctx, $args, $cond ) = @_;
    return unless $ctx;
    my $tag = $ctx->stash( 'tag' );
    $tag =~ s/^link//i;
    $tag =~ s/on$//i;
    $tag =lc( $tag ) . '_on';
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    my $date = $link->$tag;
    $args->{ ts } = $date;
    $date = $ctx->build_date( $args );
    return $date || '';
}

sub _hdlr_if_link_broken {
    my ( $ctx, $args, $cond ) = @_;
    return unless $ctx;
    my $tag = $ctx->stash( 'tag' );
    $tag =lc( $tag );
    $tag =~ s/^ifbroken/broken_/i;
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    my $bool = $link->$tag;
    return $bool if $bool;
    return 0;
}

sub _hdlr_if_link_active {
    my ( $ctx, $args, $cond ) = @_;
    return unless $ctx;
    my $tag = $ctx->stash( 'tag' );
    $tag =lc( $tag );
    $tag =~ s/^ifactive/broken_/i;
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    my $bool = $link->$tag;
    return 0 if ( $bool == 1 );
    return 1;
}

sub _hdlr_author_displayname {
    my ( $ctx, $args, $cond ) = @_;
    my $link = $ctx->stash( 'link' );
    return $ctx->error() unless defined $link;
    my $author_name = $link->author->nickname;
    $author_name = $link->author->name unless $author_name;
    return $author_name;
}

sub _list_tag {
    my ( $cb, $app, $param, $tmpl ) = @_;
    if ( $app->param( 'filter_key' ) eq 'link' ) {
        $param->{ filter_label } = $plugin->translate( 'Tags with links' );
        $param->{ screen_group } = 'link';
    }
    my $list_filters = $param->{ list_filters };
            push @$list_filters,
            {
              key   => 'link',
              label => $plugin->translate( 'Tags with links' ),
            };
    $param->{ list_filters } = $list_filters;
}

sub _list_link {
    my $app = shift;
    my $user = $app->user;
    my $mode = $app->mode;
    my $list_id = $mode;
    $list_id =~ s/^list_//;
    my %blogs;
    my $system_view;
    my $website_view;
    my $blog_view;
    my @blog_ids;
    my $r = MT::Request->instance;
    if (! defined $app->blog ) {
        $system_view = 1;
        my @all_blogs = MT::Blog->load( { class => [ 'website', 'blog' ] } );
        for my $blog ( @all_blogs ) {
            if ( _link_permission( $blog ) ) {
                $blogs{ $blog->id } = $blog;
            }
        }
    } else {
        if ( $app->param( 'dialog_view' ) ) {
            if (! _post_permission( $app->blog ) ) {
                return $app->trans_error( 'Permission denied.' );
            }
        } else {
            if (! _link_permission( $app->blog ) ) {
                return $app->trans_error( 'Permission denied.' );
            }
        }
        if ( $app->blog->class eq 'website' ) {
            $website_view = 1;
            $blogs{ $app->blog->id } = $app->blog;
            push ( @blog_ids, $app->blog->id );
            my @all_blogs = MT::Blog->load( { parent_id => $app->blog->id } );
            for my $blog ( @all_blogs ) {
                if ( _link_permission( $blog ) ) {
                    $blogs{ $blog->id } = $blog;
                    push ( @blog_ids, $blog->id );
                }
            }
        } else {
            $blog_view = 1;
            push ( @blog_ids, $app->blog->id );
        }
    }
    my $code = sub {
        my ( $obj, $row ) = @_;
        my $columns = $obj->column_names;
        for my $column ( @$columns ) {
            my $val = $obj->$column;
            if ( $column =~ /_on$/ ) {
                $val = format_ts( "%Y&#24180;%m&#26376;%d&#26085;", $val, undef,
                                  $app->user ? $app->user->preferred_language : undef );
            } else {
                if ( $column eq 'description' ) {
                    $val = substr_text( $val, 0, 20 ) . ( length_text( $val ) > 20 ? "..." : "" );
                } elsif ( $column eq 'url' ) {
                    my $short = substr_text( $val, 0, 40 ) . ( length_text( $val ) > 40 ? "..." : "" );
                    $row->{ 'url_short' } = $short;
                }
            }
            $row->{ $column } = $val;
        }
        if ( (! defined $app->blog ) || ( $website_view ) ) {
            if ( defined $blogs{ $obj->blog_id } ) {
                my $blog_name = $blogs{ $obj->blog_id }->name;
                $blog_name = substr_text( $blog_name, 0, 20 ) . ( length_text( $blog_name ) > 20 ? "..." : "" );
                $row->{ weblog_name } = $blog_name;
                $row->{ weblog_id } = $obj->blog_id;
                $row->{ can_edit } = _link_permission( $blogs{ $obj->blog_id } );
            }
        } else {
            $row->{ can_edit } = 1;
        }
        my $obj_author = $obj->author;
        $row->{ author_name } = $obj_author->name;
    };
    my @link_admin = _load_link_admin( @blog_ids );
    my @author_loop;
    for my $admin ( @link_admin ) {
        $r->cache( 'cache_author:' . $admin->id, $admin );
        push @author_loop, {
                author_id => $admin->id,
                author_name => $admin->name, };
    }
    my %terms;
    my %param;
    my @tag_loop;
    require MT::Tag;
    require MT::ObjectTag;
    my @tags = MT::Tag->load( undef,
                              { join => MT::ObjectTag->join_on( 'tag_id',
                              { blog_id => \@blog_ids, object_datasource => 'link', },
                              { unique => 1, } ) } );
    for my $tag ( @tags ) {
        push @tag_loop, {
            tag_name => $tag->name, };
    }
    $param{ tag_loop } = \@tag_loop;
    $param{ list_id } = $list_id;
    $param{ dialog_view }  = $app->param( 'dialog_view' );
    $param{ edit_field }   = $app->param( 'edit_field' );
    $param{ author_loop }  = \@author_loop;
    $param{ system_view }  = $system_view;
    $param{ website_view } = $website_view;
    $param{ filter } = $app->param( 'filter' );
    $param{ filter_val } = $app->param( 'filter_val' );
    $param{ blog_view }    = $blog_view;
    $param{ LIST_NONCRON } = 1;
    $param{ saved_deleted } = 1 if $app->param( 'saved_deleted' );
    $param{ published }      = 1 if $app->param( 'published' );
    $param{ not_published }  = 1 if $app->param( 'not_published' );
    $param{ unpublished }    = 1 if $app->param( 'unpublished' );
    $param{ not_unpublished }= 1 if $app->param( 'not_published' );
    $param{ imported } = 1 if $app->param( 'imported' );
    $param{ not_imported } = 1 if $app->param( 'not_imported' );
    $param{ broken_links } = $app->param( 'broken_links' );
    $param{ broken_rsses } = $app->param( 'broken_rsses' );
    $param{ broken_images }= $app->param( 'broken_images' );
    $param{ broken } = $app->param( 'broken' );
    $param{ not_broken } = $app->param( 'not_broken' );
    $param{ link_checked } = $app->param( 'link_checked' );
    $param{ not_link_checked } = $app->param( 'not_link_checked' );
    if ( $website_view ) {
        $terms{ blog_id } = \@blog_ids;
    }
    if ( $app->param( 'filter' ) eq 'rating' ) {
        if ( $app->param( 'filter_val' ) == '0' ) {
            $terms{ rating } = 0;
        }
    }
    $app->{ plugin_template_path } = File::Spec->catdir( $plugin->path, 'tmpl' );
    if ( my $query = $app->param( 'query' ) ) {
        my $search_col = $app->param( 'search_col' );
        $param{ query } = $query;
        $param{ filter } = $query;
        $param{ search_col } = $search_col;
        my %terms1 = ( blog_id => \@blog_ids,
                       $search_col => { like => "%$query%" }, );
        return $app->listing (
            {
                type   => $list_id,
                code   => $code,
                args   => { sort => 'authored_on', direction => 'descend' },
                params => \%param,
                terms  => \%terms1,
            }
        );
    }
    return $app->listing (
        {
            type   => $list_id,
            code   => $code,
            args   => { sort => 'authored_on', direction => 'descend' },
            params => \%param,
            terms  => \%terms,
        }
    );
}

sub _upload_link {
    my $app = shift;
    my $user = $app->user;
    my $blog = $app->blog;
    if (! defined $blog ) {
        $app->return_to_dashboard();
    }
    if (! _link_permission( $blog ) ) {
        return $app->trans_error( 'Permission denied.' );
    }
    $app->validate_magic or return $app->trans_error( 'Permission denied.' );
    my $tempdir = $app->config( 'TempDir' );
    my $workdir = tempdir ( DIR => $tempdir );
    my $upload = upload( 'file', $workdir );
    my $do;
    my $model = $app->model( 'link' );
    my $ts = current_ts( $blog );
    if ( file_extension ( $upload ) eq 'xml' ) {
        require XML::Simple;
        eval {
            local( $^W ) = 0;
            require XML::Parser;
        };
        unless ( $@ ) {
            $XML::Simple::PREFERRED_PARSER = 'XML::Parser';
        }
        my $xml = XML::Simple->new();
        my $opml = $xml->XMLin( $upload );
        my $body = $opml->{ body };
        my $outline = $body->{ outline };
        $outline = $outline->{ outline };
        for my $item ( @$outline ) {
            if ( $item->{ htmlUrl } ) {
                my $link = $model->new;
                $link->name( utf8_on( $item->{ title } ) );
                $link->url ( utf8_on( $item->{ htmlUrl } ) );
                $link->rss_address( utf8_on( $item->{ xmlUrl } ) );
                $link->author_id( $app->user->id );
                $link->blog_id( $blog->id );
                $link->authored_on( $ts );
                $link->created_on( $ts );
                $link->modified_on( $ts );
                $link->status( 1 );
                $link->rating( 0 );
                $link->save or die $link->errstr;
                $do = 1;
            }
        }
    }
    File::Path::rmtree( [ $workdir ] );
    if ( $do ) {
        $app->add_return_arg( imported => 1 );
    } else {
        $app->add_return_arg( not_imported => 1 );
    }
    $app->call_return;
}

sub _publish_links {
    _status_change( 'published', Link::Link::RELEASE() );
}

sub _unpublish_links {
    _status_change( 'unpublished', Link::Link::HOLD() );
}

sub _status_change {
    my ( $param, $status ) = @_;
    my $app = MT::instance();
    my @id = $app->param( 'id' );
    my $do;
    for my $link_id ( @id ) {
        my $link = $app->model( 'link' )->load( $link_id );
        return $app->errtrans( 'Invalid request.' ) unless $link;
        if (! _link_permission( $link->blog ) ) {
            return $app->trans_error( 'Permission denied.' );
        }
        if ( $link->status != $status ) {
            my $original = $link->clone_all();
            my $before = $plugin->translate( $original->status_text );
            $link->status( $status );
            $link->save or die $link->errstr;
            if ( $status == Link::Link::HOLD() ) {
                $app->run_callbacks( 'post_unpublish.link', $app, $link, $original );
            } elsif ( $status == Link::Link::RELEASE() ) {
                $app->run_callbacks( 'post_publish.link', $app, $link, $original );
            }
            my $after = $plugin->translate( $link->status_text );
            $app->log( {
                message => $plugin->translate( 'Link \'[_1]\' (ID:[_2]) edited and its status changed from [_3] to [_4] by user \'[_5]\'', $link->name, $link->id, $before, $after, $app->user->name ),
                blog_id => $link->blog_id,
                author_id => $app->user->id,
                class => 'link',
                level => MT::Log::INFO(),
            } );
            $do = 1;
        }
    }
    if ( $do ) {
        $app->add_return_arg( $param => 1 );
    } else {
        $app->add_return_arg( 'not_' . $param => 1 );
    }
    $app->call_return;
}

sub _action_link_check {
    my $app = shift;
    if (! defined $app->blog ) {
        $app->return_to_dashboard( redirect => 1 );
    } else {
        if (! _link_permission( $app->blog ) ) {
            return $app->trans_error( 'Permission denied.' );
        }
    }
    my @id = $app->param( 'id' );
    my $do;
    my $broken_links = 0;
    my $broken_rsses = 0;
    my $broken_images = 0;
    require LWP::UserAgent;
    require HTTP::Date;
    require Digest::MD5;
    my $remote_ip = $app->remote_ip;
    my $agent = "Mozilla/5.0 (Movable Type Link plugin X_FORWARDED_FOR:$remote_ip)";
    my $ua = LWP::UserAgent->new( agent => $agent );
    for my $link_id ( @id ) {
        my $obj = MT->model( 'link' )->load( $link_id );
        my $change;
        if ( $obj->url ) {
            my $response = $ua->head( $obj->url );
            if (! $response->is_success ) {
                $change = 1 if (! $obj->broken_link );
                $obj->broken_link( 1 );
                $obj->urlupdated_on( undef );
                $broken_links++;
            } else {
                my $content = get_content( $obj->url );
                if ( $content ) {
                    $content = Digest::MD5::md5_hex( $content );
                    if ( (! $obj->digest ) || ( $obj->digest ne $content ) ) {
                        my ( $year, $mon, $day, $hour, $min, $sec, $tz ) = HTTP::Date::parse_date( $response->header( "Last-Modified" ) );
                        my $modified = sprintf( "%04d%02d%02d%02d%02d%02d", $year, $mon, $day, $hour, $min, $sec );
                        $obj->urlupdated_on( $modified );
                        $obj->digest( $content );
                        $change = 1;
                    }
                }
                $change = 1 if ( $obj->broken_link );
                $obj->broken_link( 0 );
            }
        } else {
            $change = 1 if (! $obj->broken_link );
            $obj->broken_link( 1 );
            $obj->urlupdated_on( undef );
        }
        if ( $obj->rss_address ) {
            my $response = $ua->head( $obj->rss_address );
            if (! $response->is_success ) {
                $change = 1 if (! $obj->broken_rss );
                $obj->broken_rss( 1 );
                $obj->rssupdated_on( undef );
                $broken_rsses++;
            } else {
                my $content = get_content( $obj->rss_address );
                my $modified;
                if ( ( $content ) && ( $content =~ m!^.*?<.*?date.*?>(.*?)</.*?date.*?>!si ) ) {
                    $modified = $1;
                } else {
                    $modified = $response->header( "Last-Modified" );
                }
                my ( $year, $mon, $day, $hour, $min, $sec, $tz ) = HTTP::Date::parse_date( $modified );
                $modified = sprintf( "%04d%02d%02d%02d%02d%02d", $year, $mon, $day, $hour, $min, $sec );
                if ( ( ! $obj->rssupdated_on ) || ( $obj->rssupdated_on ne $modified ) ) {
                    $obj->rssupdated_on( $modified );
                    $change = 1;
                }
                $change = 1 if ( $obj->broken_rss );
                $obj->broken_rss( 0 );
            }
        } else {
            $change = 1 if ( $obj->broken_rss );
            $obj->broken_rss( 0 );
        }
        if ( $obj->image_address ) {
            my $response = $ua->head( $obj->image_address );
            if (! $response->is_success ) {
                $change = 1 if (! $obj->broken_image );
                $obj->broken_image( 1 );
                $broken_images++;
            } else {
                $change = 1 if ( $obj->broken_image );
                $obj->broken_image( 0 );
            }
        } else {
            $change = 1 if ( $obj->broken_image );
            $obj->broken_image( 0 );
        }
        if ( $change ) {
            $obj->save or die $obj->errstr;
        }
        $do = 1;
    }
    if ( $do ) {
        $app->add_return_arg( link_checked => 1 );
        $app->add_return_arg( broken_links => $broken_links );
        $app->add_return_arg( broken_rsses => $broken_rsses );
        $app->add_return_arg( broken_images => $broken_images );
        if ( $broken_links || $broken_rsses || $broken_images ) {
            $app->add_return_arg( broken => 1 );
        } else {
            $app->add_return_arg( not_broken => 1 );
        }
    } else {
        $app->add_return_arg( not_link_checked => 1 );
    }
    $app->call_return;
}

sub _task_link_check {
    my $app = MT->instance();
    my $iter = MT::Blog->load_iter( { class => '*' } );
    require LWP::UserAgent;
    require HTTP::Date;
    require Digest::MD5;
    my $agent = "Mozilla/5.0 (Movable Type Link plugin)";
    my $ua = LWP::UserAgent->new( agent => $agent );
    my $do;
    while ( my $blog = $iter->() ) {
        my $blog_do;
        my $error;
        my $check_outlink = $plugin->get_config_value( 'check_outlink', 'blog:'. $blog->id );
        next unless $check_outlink;
        my @links = MT->model( 'link' )->load( { blog_id => $blog->id } );
        for my $obj ( @links ) {
            my $original = $obj->clone_all;
            my $change;
            if ( $obj->url ) {
                $do = 1; $blog_do = 1;
                my $response = $ua->head( $obj->url );
                if (! $response->is_success ) {
                    $change = 1 if (! $obj->broken_link );
                    $obj->broken_link( 1 );
                    $obj->urlupdated_on( undef );
                    $app->log( {
                        message => $plugin->translate( 'This URL \'[_1]\' is broken.', $obj->url ),
                        blog_id => $obj->blog_id,
                        class => 'link',
                        level => MT::Log::ERROR(),
                    } );
                    $error = 1;
                    $app->run_callbacks( 'post_broken_url.link', $app, $obj, $original );
                } else {
                    my $content = get_content( $obj->url );
                    if ( $content ) {
                        $content = Digest::MD5::md5_hex( $content );
                        if ( (! $obj->digest ) || ( $obj->digest ne $content ) ) {
                            my ( $year, $mon, $day, $hour, $min, $sec, $tz ) = HTTP::Date::parse_date( $response->header( "Last-Modified" ) );
                            my $modified = sprintf( "%04d%02d%02d%02d%02d%02d", $year, $mon, $day, $hour, $min, $sec );
                            $obj->urlupdated_on( $modified );
                            $obj->digest( $content );
                            $change = 1;
                        }
                    }
                    $change = 1 if ( $obj->broken_link );
                    $obj->broken_link( 0 );
                }
            } else {
                $change = 1 if (! $obj->broken_link );
                $obj->broken_link( 1 );
                $obj->urlupdated_on( undef );
            }
            if ( $obj->rss_address ) {
                $do = 1; $blog_do = 1;
                my $response = $ua->head( $obj->rss_address );
                if (! $response->is_success ) {
                    $change = 1 if (! $obj->broken_rss );
                    $obj->broken_rss( 1 );
                    $obj->rssupdated_on( undef );
                    $app->log( {
                        message => $plugin->translate( 'This RSS \'[_1]\' is broken.', $obj->rss_address ),
                        blog_id => $obj->blog_id,
                        class => 'link',
                        level => MT::Log::ERROR(),
                    } );
                    $error = 1;
                    $app->run_callbacks( 'post_broken_rss.link', $app, $obj, $original );
                } else {
                    my $content = get_content( $obj->rss_address );
                    my $modified;
                    if ( ( $content ) && ( $content =~ m!^.*?<.*?date.*?>(.*?)</.*?date.*?>!si ) ) {
                        $modified = $1;
                    } else {
                        $modified = $response->header( "Last-Modified" );
                    }
                    my ( $year, $mon, $day, $hour, $min, $sec, $tz ) = HTTP::Date::parse_date( $modified );
                    $modified = sprintf( "%04d%02d%02d%02d%02d%02d", $year, $mon, $day, $hour, $min, $sec );
                    if ( ( ! $obj->rssupdated_on ) || ( $obj->rssupdated_on ne $modified ) ) {
                        $obj->rssupdated_on( $modified );
                        $change = 1;
                    }
                    $change = 1 if ( $obj->broken_rss );
                    $obj->broken_rss( 0 );
                }
            } else {
                $change = 1 if ( $obj->broken_rss );
                $obj->broken_rss( 0 );
            }
            if ( $obj->image_address ) {
                $do = 1; $blog_do = 1;
                my $response = $ua->head( $obj->image_address );
                if (! $response->is_success ) {
                    $change = 1 if (! $obj->broken_image );
                    $obj->broken_image( 1 );
                    $app->log( {
                        message => $plugin->translate( 'This image \'[_1]\' is broken.', $obj->image_address ),
                        blog_id => $obj->blog_id,
                        class => 'link',
                        level => MT::Log::ERROR(),
                    } );
                    $error = 1;
                    $app->run_callbacks( 'post_broken_image.link', $app, $obj, $original );
                } else {
                    $change = 1 if ( $obj->broken_image );
                    $obj->broken_image( 0 );
                }
            } else {
                $change = 1 if ( $obj->broken_image );
                $obj->broken_image( 0 );
            }
            if ( $change ) {
                $obj->save or die $obj->errstr;
            }
        }
        if ( $blog_do ) {
            $app->run_callbacks( 'post_task_blog_linkcheck', $app, $blog, $error );
        }
    }
    if ( $do ) {
        return 1;
    } else {
        return 0;
    }
}

sub _search_link  {
    my $app = shift;
    my ( %args ) = @_;
    my %blogs;
    my $system_view;
    my $website_view;
    my $blog_view;
    my @blog_ids;
    my $r = MT::Request->instance;
    if (! defined $app->blog ) {
        $app->return_to_dashboard( redirect => 1 );
    } else {
        if (! _link_permission( $app->blog ) ) {
            return $app->trans_error( 'Permission denied.' );
        }
        if ( $app->blog->class eq 'website' ) {
            $website_view = 1;
            $blogs{ $app->blog->id } = $app->blog;
            push ( @blog_ids, $app->blog->id );
            my @all_blogs = MT::Blog->load( { parent_id => $app->blog->id } );
            for my $blog ( @all_blogs ) {
                if ( _link_permission( $blog ) ) {
                    $blogs{ $blog->id } = $blog;
                    push ( @blog_ids, $blog->id );
                }
            }
            push ( @all_blogs, $app->blog );
        } else {
            $blog_view = 1;
            push ( @blog_ids, $app->blog->id );
        }
    }
    my $iter;
    if ( $args{ iter } ) {
        $iter = $args{ iter };
    } elsif ( $args{ items } ) {
        $iter = sub { pop @{ $args{ items } } };
    }
    return [] unless $iter;
    my $limit = $args{ limit };
    my $param = $args{ param } || {};
    my @data;
    while ( my $obj = $iter->() ) {
        my $row = $obj->column_values;
        $row->{ object } = $obj;
        my $columns = $obj->column_names;
        for my $column ( @$columns ) {
            my $val = $obj->$column;
            if ( $column =~ /_on$/ ) {
                $val = format_ts( "%Y&#24180;%m&#26376;%d&#26085;", $val, undef,
                                  $app->user ? $app->user->preferred_language : undef );
            } else {
                if ( $column eq 'description' ) {
                    $val = substr_text( $val, 0, 20 ) . ( length_text( $val ) > 20 ? "..." : "" );
                } elsif ( $column eq 'url' ) {
                    my $short = substr_text( $val, 0, 40 ) . ( length_text( $val ) > 40 ? "..." : "" );
                    $row->{ 'url_short' } = $short;
                }
            }
            $row->{ $column } = $val;
        }
        if ( (! defined $app->blog ) || ( $website_view ) ) {
            if ( defined $blogs{ $obj->blog_id } ) {
                my $blog_name = $blogs{ $obj->blog_id }->name;
                $blog_name = substr_text( $blog_name, 0, 20 ) . ( length_text( $blog_name ) > 20 ? "..." : "" );
                $row->{ weblog_name } = $blog_name;
                $row->{ weblog_id } = $obj->blog_id;
                $row->{ can_edit } = _link_permission( $blogs{ $obj->blog_id } );
            }
        } else {
            $row->{ can_edit } = 1;
        }
        my $obj_author = $obj->author;
        $row->{ author_name } = $obj_author->name;
        push @data, $row;
        last if $limit and @data > $limit;
    }
    $param->{ search_label } = $plugin->translate( 'Link' );
    $param->{ search_replace } = 1;
    return [] unless @data;
    $param->{ system_view } = 1 unless $app->param ( 'blog_id' );
    $param->{ object_loop } = \@data;
    \@data;
}

sub _edit_link {
    my ( $cb, $app, $param, $tmpl ) = @_;
    my $type  = $app->param( '_type' );
    my $class = $app->model( $type );
    my $blog  = $app->blog;
    if (! $blog ) {
        $app->return_to_dashboard();
    }
    if (! _link_permission( $app->blog ) ) {
        $app->return_to_dashboard( permission => 1 );
    }
    my $id = $app->param( 'id' );
    if ( $id ) {
        my $obj = $class->load( $id );
        if (! defined $obj ) {
            $app->return_to_dashboard( permission => 1 );
        }
        if ( $obj->blog_id != $blog->id ) {
            $app->return_to_dashboard( permission => 1 );
        }
        my @tags = $obj->tags;
        my $tag = join( ',', @tags );
        $param->{ tags } = $tag;
        my $next = $obj->_nextprev( 'next' );
        if ( $next ) {
            $param->{ next_link_id } = $next->id;
        }
        my $previous = $obj->_nextprev( 'previous' );
        if ( $previous ) {
            $param->{ previous_link_id } = $previous->id;
        }
        my $columns = $obj->column_names;
        if (! $app->param( 'saved' ) ) {
            my $check_outlink = $plugin->get_config_value( 'check_outlink', 'blog:'. $app->blog->id );
            my $ua;
            require LWP::UserAgent;
            if ( $check_outlink ) {
                my $remote_ip = $app->remote_ip;
                my $agent = "Mozilla/5.0 (Movable Type Link plugin X_FORWARDED_FOR:$remote_ip)";
                $ua = LWP::UserAgent->new( agent => $agent );
                my $change;
                if ( $obj->url ) {
                    my $response = $ua->head( $obj->url );
                    if (! $response->is_success ) {
                        $change = 1 if (! $obj->broken_link );
                        $obj->broken_link( 1 );
                    } else {
                        $change = 1 if ( $obj->broken_link );
                        $obj->broken_link( 0 );
                    }
                } else {
                    $change = 1 if (! $obj->broken_link );
                    $obj->broken_link( 1 );
                }
                if ( $obj->rss_address ) {
                    my $response = $ua->head( $obj->rss_address );
                    if (! $response->is_success ) {
                        $change = 1 if (! $obj->broken_rss );
                        $obj->broken_rss( 1 );
                    } else {
                        $change = 1 if ( $obj->broken_rss );
                        $obj->broken_rss( 0 );
                    }
                } else {
                    $change = 1 if ( $obj->broken_rss );
                    $obj->broken_rss( 0 );
                }
                if ( $obj->image_address ) {
                    my $response = $ua->head( $obj->image_address );
                    if (! $response->is_success ) {
                        $change = 1 if (! $obj->broken_image );
                        $obj->broken_image( 1 );
                    } else {
                        $change = 1 if ( $obj->broken_image );
                        $obj->broken_image( 0 );
                    }
                } else {
                    $change = 1 if ( $obj->broken_image );
                    $obj->broken_image( 0 );
                }
                if ( $change ) {
                    $obj->save or die $obj->errstr;
                }
            }
        }
        for my $column ( @$columns ) {
            if ( $column =~ /_on$/ ) {
                my $column_ts = $obj->$column;
                $param->{ $column . '_date' } = format_ts( '%Y-%m-%d', $column_ts );
                $param->{ $column . '_time' } = format_ts( '%H:%M:%S', $column_ts );
            }
        }
    } else {
        my $columns = $class->column_names;
        my @tl = offset_time_list( time, $app->blog );
        my $ts_date = sprintf "%04d-%02d-%02d", $tl[5]+1900, $tl[4]+1, $tl[3];
        my $ts_time = sprintf "%02d:%02d:%02d",  @tl[2,1,0];
        for my $column ( @$columns ) {
            if ( $column =~ /_on$/ ) {
                $param->{ $column . '_date' } = $ts_date;
                $param->{ $column . '_time' } = $ts_time;
            }
        }
    }
    my %args = ( blog => $app->blog );
    $param->{ saved } = $app->param( 'saved' );
    $param->{ search_label } = $plugin->translate( 'Link' );
    $param->{ screen_group } = 'link';
    $param->{ return_args } = _force_view_mode_return_args( $app );
}

sub _archetype_editor {
    my ( $cb, $app, $tmpl ) = @_;
    my $pointer = quotemeta( 'Insert File</b><s></s></a>' );
    my $src =<<'MTML';
                                <__trans_section component="Link">
                                    <a href="<mt:var name="script_url">?__mode=list_link&amp;edit_field=<mt:var name="toolbar_edit_field">&amp;blog_id=<mt:var name="blog_id">&amp;dialog_view=1" title="<__trans phrase="Insert Link" escape="html">" class="command-insert-link toolbar button mt-open-dialog"><b>Insert Link</b><s></s></a>&nbsp;&nbsp;&nbsp;
                                </__trans_section>
MTML
    $$tmpl =~ s/($pointer)/$1$src/;
}

sub _load_link_admin {
    my @blog_id = @_;
    push ( @blog_id, 0 );
    my $author_class = MT->model( 'author' );
    my %terms1 = ( blog_id => \@blog_id, permissions => { like => "\%'administer\%" } );
    my @admin = $author_class->load(
        { type => MT::Author::AUTHOR(), },
        { join => [ 'MT::Permission', 'author_id',
            \%terms1,
            { unique => 1 } ],
        }
    );
    my @author_id;
    for my $author ( @admin ) {
        push ( @author_id, $author->id );
    }
    my %terms2 = ( blog_id => \@blog_id, permissions => { like => "\%'manage_link'\%" } );
    my @link_admin = $author_class->load(
        { type => MT::Author::AUTHOR(),
          id => { not => \@author_id } },
        { join => [ 'MT::Permission', 'author_id',
            \%terms2,
            { unique => 1 } ],
        }
    );
    push ( @admin, @link_admin );
    return @admin;
}

sub _link_permission {
    my ( $blog, $user ) = @_;
    my $app = MT->instance();
    $user = $app->user unless $user;
    return 1 if $user->is_superuser;
    $blog = $app->blog unless $blog;
    if ( is_user_can( $blog, $user, 'administer_blog' ) ) {
        return 1;
    }
    if ( is_user_can( $blog, $user, 'manage_link' ) ) {
        return 1;
    }
    return 0;
}

sub _post_permission {
    my $blog = shift;
    my $app = MT->instance();
    my $user = $app->user;
    return 1 if $user->is_superuser;
    $blog = $app->blog unless $blog;
    if ( is_user_can( $blog, $user, 'administer_blog' ) ) {
        return 1;
    }
    if ( is_user_can( $blog, $user, 'manage_pages' ) ) {
        return 1;
    }
    if ( is_user_can( $blog, $user, 'create_post' ) ) {
        return 1;
    }
    return 0;
}

sub _install_role {
    my $app = MT->instance();
    require MT::Role;
    my $role = MT::Role->get_by_key( { name => $plugin->translate( 'Link Administrator' ) } );
    if (! $role->id ) {
        my $role_en = MT::Role->load( { name => 'Link Administrator' } );
        if (! $role_en ) {
            my %values;
            $values{ created_by }  = $app->user->id;
            $values{ description } = $plugin->translate( 'Can create Link, edit Link.' );
            $values{ is_system } = 0;
            $values{ permissions } = "'manage_link'";
            $role->set_values( \%values );
            $role->save
                or return $app->trans_error( 'Error saving role: [_1]', $role->errstr );
        }
    }
    return 1;
}

sub _force_view_mode_return_args {
    my $app = shift;
    my $return = $app->make_return_args;
    $return =~ s/edit/view/;
    return $return;
}

sub is_user_can {
    my ( $blog, $user, $permission ) = @_;
    $permission = 'can_' . $permission;
    my $perm = $user->is_superuser;
    unless ( $perm ) {
        if ( $blog ) {
            $perm = $user->permissions( $blog->id )->$permission;
        }
    }
    return $perm;
}

sub upload {
    my ( $name, $dir ) = @_;
    my $app = MT->instance();
    require MT::FileMgr;
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    my $q = $app->param;
    my $file = $q->upload( $name );
    my $file_extension = file_extension( $file );
    my $out = File::Spec->catfile( $dir, 'import.' . $file_extension );
    $dir =~ s!/$!! unless $dir eq '/';
    unless ( $fmgr->exists( $dir ) ) {
        $fmgr->mkpath( $dir ) or return MT->trans_error( "Error making path '[_1]': [_2]",
                                $out, $fmgr->errstr );
    }
    my $temp = "$out.new";
    local *OUT;
    open ( OUT, ">$out" ) or die "Can't open $out!";
    binmode ( OUT );
    while( read ( $file, my $buffer, 1024 ) ) {
        $buffer=~ s/\r\n/\n/gs;
        $buffer =~ s/\r/\n/g;
        print OUT $buffer;
    }
    close ( OUT );
    $fmgr->rename( $temp, $out );
    return $out;
}

sub utf8_on {
    my $text = shift;
    if (! Encode::is_utf8( $text ) ) {
        Encode::_utf8_on( $text );
    }
    return $text;
}

sub get_weblog_ids {
    my $blog = shift;
    my @blog_ids;
    push ( @blog_ids, $blog->id );
    if ( $blog->class eq 'website' ) {
        my @blogs = MT::Blog->load( { parent_id => $blog->id } );
        for my $weblog ( @blogs ) {
            push ( @blog_ids, $weblog->id );
        }
    }
    return @blog_ids;
}

sub current_ts {
    my $blog = shift;
    my @tl = offset_time_list( time, $blog );
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
    return $ts;
}

sub valid_ts {
    my $ts = shift;
    return 0 unless ( $ts =~ m/^[0-9]{14}$/ );
    my $year = substr( $ts, 0, 4 );
    my $month = substr( $ts, 4, 2 );
    my $day = substr( $ts, 6, 2 );
    my ( @mlast ) = ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
    if ( $month < 1 || 12 < $month ) {
        return 0;
    }
    if ( $month == 2 ) {
        if ( ( ( $year % 4 == 0 ) && ( $year % 100 != 0 ) ) || ( $year % 400 == 0 ) ) {
            $mlast[1]++;
        }
    }
    if ( $day < 1 || $mlast[$month-1] < $day ) {
        return 0;
    }
    my $hour = substr( $ts, 8, 2 );
    my $min = substr( $ts, 10, 2 );
    my $sec = substr( $ts, 12, 2 );
    if ( ( $hour < 25 ) && ( $min < 61 ) && ( $sec < 61 ) ) {
        return 1;
    } else {
        return 0;
    }
}

sub get_content {
    my ( $uri, $utf8, $file ) = @_;
    my $app = MT->instance();
    eval { require LWP::UserAgent };
    if ( $@ ){
        return undef;
    }
    my $remote_ip;
    eval { $remote_ip = $app->remote_ip; };
    my $agent;
    if ( $remote_ip ) {
        $agent = "Mozilla/5.0 (Movable Type Link plugin X_FORWARDED_FOR:$remote_ip)";
    } else {
        $agent = "Mozilla/5.0 (Movable Type Link plugin)";
    }
    my $protcol = $1 if $uri =~ /(^.*?):/;
    my $ua = LWP::UserAgent->new;
    $ua->agent( $agent );
    my $proxy = MT->instance->config->PingProxy;
    $ua->proxy( $protcol, $proxy ) if ( $protcol && $proxy );
    my $no_proxy = MT->instance->config->PingNoProxy;
    $ua->no_proxy( split( /,\s*/, $no_proxy ) ) if $no_proxy;
    my $timeout = MT->instance->config->PingTimeout;
    $ua->timeout( $timeout ) if $timeout;
    my $req = HTTP::Request->new( GET => $uri );
    my $res = $ua->request( $req );
    if ( $res ) {
        my $content = $res->content;
        $content = MT::I18N::encode_text( $content, undef, 'utf-8' );
        $content = utf8_on( $content ) if $utf8;
        return $content;
    } else {
        return undef;
    }
    return undef;
}

sub file_extension {
    my ( $file ) = @_;
    my $extension;
    if ( $file =~ /\.([^\.]*)$/ ) {
        $extension = $1;
        $extension = lc ( $extension );
        return $extension;
    } else {
        return '';
    }
}

1;